#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import datetime
import json
import logging
import re
import requests
import time
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

logging.basicConfig(format='%(asctime)s %(levelname)-8s %(message)s', level=logging.ERROR, datefmt='%Y-%m-%d %H:%M:%S')

class vvx():

    """
    Class for VVX based on UCS 6.1.0 REST APIs.
    INPUTS for instance creation:
        - ipaddr = IP address of phone
        - auth = username and password in tuple
    Attributes:
        .ipaddr
        .auth_credentials
        .use_https
        .verify_secure
        .model
        .firmware
        .macaddress
        .baseprofile
    
    Methods:
       .getDeviceInfoV2() - calls "/api/v2/mgmt/device/info"
       .getNetwork() - calls "/api/v1/mgmt/network/info"
       .getLineInfoV2() - calls "/api/v2/mgmt/lineInfo"
       .getCallStatusV2() - calls "/api/v2/webCallControl/callStatus"
       .getRunningConfig() - calls "/api/v1/mgmt/device/runningConfig"
       .getDeviceStats() - calls "/api/v1/mgmt/device/stats"
       .getNetworkStats() - calls "/api/v1/mgmt/network/stats"
       .getSessionStats() - calls "/api/v1/mgmt/media/sessionStats"
       .getCallLogs() - calls "/api/v1/mgmt/callLogs"
       .getConfig() - calls "/api/v1/mgmt/config/get"
       .setConfig() - calls "/api/v1/mgmt/config/set"
       .callDial() - "/api/v1/callctrl/dial"
       .callEnd() - "/api/v1/callctrl/endCall"
       .simulateKeyEvent() - calls "/api/v1/mgmt/simulateKeyEvent"
       .simulateTextInput() - calls "/api/v1/mgmt/simulateTextInput"
       .safeRestart() - calls "/api/v1/mgmt/safeRestart"
       .safeReboot() - calls "/api/v1/mgmt/safeReboot"
       .factoryReset() - calls "/api/v1/mgmt/factoryReset"
       .updateConfig() - calls "/api/v1/mgmt/updateConfiguration"
       .resetConfig() - calls "/api/v1/mgmt/configReset"
    """

    
    __qpaths_dict={

            "sipStatus" : "/api/v1/webCallControl/sipStatus",
            "network" : "/api/v1/mgmt/network/info", 
            "deviceinfov2" : "/api/v2/mgmt/device/info",
            "lineinfov2" : "/api/v2/mgmt/lineInfo",
            "runningConfig" : "/api/v1/mgmt/device/runningConfig",
            "getconfig" : "/api/v1/mgmt/config/get",
            "setconfig" : "/api/v1/mgmt/config/set",
            "simulateTextInput" : "/api/v1/mgmt/simulateTextInput",
            "simulateKeyEvent" : "/api/v1/mgmt/simulateKeyEvent",
            "callstatusv2" : "/api/v2/webCallControl/callStatus",
            "calldial" : "/api/v1/callctrl/dial",
            "callend" : "/api/v1/callctrl/endCall",
            "callresume" : "/api/v1/callctrl/resumeCall",
            "devicestats" : "/api/v1/mgmt/device/stats",
            "networkstats" : "/api/v1/mgmt/network/stats",
            "sessionStats" : "/api/v1/mgmt/media/sessionStats",
            "callLogs" : "/api/v1/mgmt/callLogs",
            "callLogs_missed" : "/api/v1/mgmt/callLogs/missed",
            "callLogs_received" : "/api/v1/mgmt/callLogs/received",
            "callLogs_placed" : "/api/v1/mgmt/callLogs/placed",
            "safeRestart" : "/api/v1/mgmt/safeRestart",
            "safeReboot" : "/api/v1/mgmt/safeReboot",
            "factoryReset" : "/api/v1/mgmt/factoryReset",
            "updateConfig" : "/api/v1/mgmt/updateConfiguration",
            "resetConfig" : "/api/v1/mgmt/configReset",
            "resetConfig_cloud" : "/api/v1/mgmt/configReset/cloud",
            "resetConfig_local" : "/api/v1/mgmt/configReset/local",
            "resetConfig_web" : "/api/v1/mgmt/configReset/web",
            "resetConfig_device" : "/api/v1/mgmt/configReset/device"
    }
    
    _valid_versions = ("6.0.0", "6.1.0",)
    
        
    def __init__(self, ipaddr, auth_credentials, use_https=True, verify_secure=False):
        self.ipaddr = ipaddr
        self.auth_credentials = auth_credentials
        self.use_https = use_https
        self.verify_secure = verify_secure
        self.__session = requests.Session()
        
        # initiates requests.Session()
        vvx_adapter = requests.adapters.HTTPAdapter(max_retries=3)
        
        if self.use_https:
            self.__session.mount(f"https://{self.ipaddr}", vvx_adapter)
        else:
            self.__session.mount(f"http://{self.ipaddr}", vvx_adapter)

        # Extracts attributes' values for model, firmware, macaddress and _swVer
        dev = self.getDeviceInfoV2()
        if dev != None:
            self.model = dev["data"]["ModelNumber"]
            self.firmware = dev["data"]["Firmware"]["Application"]
            self.macaddress = dev["data"]["MACAddress"]
            
            for item in self._valid_versions:
                if self.firmware.startswith(item):
                    self._swVer = item
                else:
                    self._swVer = None
        else:
            self.model = None
            self.firmware = None
            self.macaddress = None            
        
        # Extracts attribute's value for baseprofile
        dev = self.getConfig({ "data" : [ "device.baseProfile" ] })
        if dev != None:
            self.baseprofile = dev["data"]["device.baseProfile"]["Value"]
        else:
            self.baseprofile = None
        
        # Extracts information about lines, count and state.
        self.lines = {}
        self.linestates = {}
        dev = self.getLineInfoV2()
        if dev != None:
            self.linescount = len(dev["data"])
            for i in range(self.linescount):
                self.lines[i+1] = dev["data"][i]["Label"]
                self.linestates[dev["data"][i]["Label"]] = dev["data"][i]["RegistrationStatus"]
        else:
            self.linescount = None

            
    def __httpRequest(self, qpath, rtype="GET", rdata={}, ctype="application/json"):
        """
        Method makes HTTP request using requests. 
        INPUTS: *qpath as string (*qpath is key value taken from qpaths_dict),
            rtype as string(GET,POST,DELETE,UPDATE, etc), ctype as string(Content-Type),
            rdata as dict(body)
        OUTPUT: Response JSON object from requests.get()
        """
        
        s = self.__session
               
        if self.use_https:
            target_url = f"https://{self.ipaddr}{self.__qpaths_dict[qpath]}"
        else:
            target_url = f"http://{self.ipaddr}{self.__qpaths_dict[qpath]}"
        
        
        try:    

            if rtype == "GET":
                r = s.get(target_url, auth=self.auth_credentials, verify=self.verify_secure, timeout=(1, 1))
                r.raise_for_status()
                return r

            elif rtype == "POST":
                headers = { "Content-Type" : ctype }
                r = s.post(url=target_url, data=json.dumps(rdata), headers=headers, 
                                  auth=self.auth_credentials, verify=self.verify_secure, timeout=(3, 3))
                r.raise_for_status()
                return r

        except requests.exceptions.HTTPError as http_err:
            logging.error( f"HTTP Error: <{http_err}>")
            time.sleep(3)
        except requests.exceptions.ConnectionError as connect_err:
            logging.error(f"Connection Error: <{connect_err}>")
        except requests.exceptions.Timeout as timeout_err:
            logging.error(f"Timeout Error: <{timeout_err}>")
        except requests.exceptions.RequestException as err:
            logging.error(f"Request Error: <{err}>")
            time.sleep(3)

            
    def getDeviceInfoV2(self):
        """
        Method calls internal httpRequest to GET "deviceinfov2" : "/api/v2/mgmt/device/info".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """     
        dev = self.__httpRequest("deviceinfov2")
        if dev != None:
            return dev.json()
        
  
    def getNetwork(self):
        """
        Method calls internal httpRequest to GET "network" : "/api/v1/mgmt/network/info".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """ 
        dev = self.__httpRequest("network")
        if dev != None:
            return dev.json()
        
    
    def getLineInfoV2(self):
        """
        Method calls internal httpRequest to GET "lineinfov2" : "/api/v2/mgmt/lineInfo".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest("lineinfov2")
        if dev != None:
            return dev.json()

            
    def getCallStatusV2(self):
        """
        Method calls internal httpRequest to GET "callstatusv2" : "/api/v2/webCallControl/callStatus".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest("callstatusv2")
        if dev != None:
            return dev.json()


    def getRunningConfig(self):
        """
        Method calls internal httpRequest to GET "runningConfig" : "/api/v1/mgmt/device/runningConfig".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest("runningConfig")
        if dev != None:
            return dev.json()

    
    def getDeviceStats(self):
        """
        Method calls internal httpRequest to GET "devicestats" : "/api/v1/mgmt/device/stats".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest("devicestats")
        if dev != None:
            return dev.json()

    def getNetworkStats(self):
        """
        Method calls internal httpRequest to GET "networkstats" : "/api/v1/mgmt/network/stats".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest("networkstats")
        if dev != None:
            return dev.json()

    def getSessionStats(self):
        """
        Method calls internal httpRequest to GET "sessionStats" : "/api/v1/mgmt/media/sessionStats".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest("sessionStats")
        if dev != None:
            return dev.json()

    def getCallLogs(self, logtype="all"):
        """
        Method calls internal httpRequest to GET "callLogs" : "/api/v1/mgmt/callLogs".
        INPUTS: logtype as str. Valid strings are "all", "missed", "received", "placed".
        OUTPUT: Returns response body as dict.
        """        
        if logtype == "missed":
            dev = self.__httpRequest("callLogs_missed")
        elif logtype == "received":
            dev = self.__httpRequest("callLogs_received")
        elif logtype == "placed":
            dev = self.__httpRequest("callLogs_placed")
        elif logtype == "all":
            dev = self.__httpRequest("callLogs")
        else:
            return logging.error(f"<Invalid input [logtype]: '{logtype}'>")
            
        if dev != None:
            return dev.json()        

        
    def safeRestart(self):
        """
        Method calls internal httpRequest to POST "safeRestart" : "/api/v1/mgmt/safeRestart".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest(qpath="safeRestart", rtype="POST")
        if dev != None:
            return dev.json()

        
    def safeReboot(self):
        """
        Method calls internal httpRequest to POST "safeReboot" : "/api/v1/mgmt/safeReboot".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest(qpath="safeReboot", rtype="POST")
        if dev != None:
            return dev.json()

        
    def factoryReset(self):
        """
        Method calls internal httpRequest to POST "factoryReset" : "/api/v1/mgmt/factoryReset".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest(qpath="factoryReset", rtype="POST")
        if dev != None:
            return dev.json()


    def updateConfig(self):
        """
        Method calls internal httpRequest to POST "updateConfig" : "/api/v1/mgmt/updateConfiguration".
        INPUTS: none
        OUTPUT: Returns response body as dict.
        """
        dev = self.__httpRequest(qpath="updateConfig", rtype="POST")
        if dev != None:
            return dev.json()

    
    def resetConfig(self, configtype="all"):
        """
        Method calls internal httpRequest to POST "resetConfig" : "/api/v1/mgmt/configReset".
        INPUTS: configtype as str. Valid strings are "all", "cloud", "local", "web", "device".
        OUTPUT: Returns response body as dict.
        """        
        if configtype == "cloud":
            dev = self.__httpRequest(qpath="resetConfig_cloud", rtype="POST")
        elif configtype == "local":
            dev = self.__httpRequest(qpath="resetConfig_local", rtype="POST")
        elif configtype == "web":
            dev = self.__httpRequest(qpath="resetConfig_web", rtype="POST")
        elif configtype == "device":
            dev = self.__httpRequest(qpath="resetConfig_device", rtype="POST")
        elif configtype == "all":
            dev = self.__httpRequest(qpath="resetConfig", rtype="POST")
        else:
            return logging.error(f"<Invalid input [configtype]: '{configtype}'>")
            
        if dev != None:
            return dev.json()        
    

    def getConfig(self, rdata, ctype="application/json"):
        """
        Method calls internal httpRequest to POST "getconfig" : "/api/v1/mgmt/config/get".
        INPUTS: ctype as string(Content-Type), rdata as dict(body)
        OUTPUT: Returns response body as dict when successful, None when unsuccessful.
        """
        # validate rdata for right structure and not empty, rdata = { "data": [...] }
        if isinstance(rdata, dict) is False:
            return logging.error("<JSON Body Error: boby passed is not a dictionary...>")
        elif "data" not in rdata:
            return logging.error("<JSON Body Error: 'data' is key missing...>")
        elif isinstance(rdata["data"], list) is False:
            return logging.error("<JSON Body Error: 'data' value is not a list...>")
        elif not rdata["data"]:
            return logging.error("<JSON Body Error: 'data' is empty...>")
        else:
            logging.debug(f"Body passed: {rdata}")
            dev = self.__httpRequest(qpath="getconfig", rtype="POST", ctype=ctype, rdata=rdata)
            if dev != None:
                return dev.json()

            
    def setConfig(self, rdata, ctype="application/json", chunk_size=20):
        """
        Method calls internal httpRequest to POST "setconfig" : "/api/v1/mgmt/config/set".
        INPUTS: ctype as string(Content-Type), rdata as dict(body)
        OUTPUT: Returns response body as list.
        """
        
        response = []
        
        # setConfig API has limit of 20 parameters in each request. 
               
        # validate rdata for right structure and not empty, rdata = { "data": {...} }
        if isinstance(rdata, dict) is False:
            return logging.error("<JSON Body Error: boby passed is not a dictionary...>")
        elif "data" not in rdata:
            return logging.error("<JSON Body Error: 'data' is key missing...>")
        elif isinstance(rdata["data"], dict) is False:
            return logging.error("<JSON Body Error: 'data' value is not a dictionary...>")
        elif not rdata["data"]:
            return logging.error("<JSON Body Error: 'data' is empty...>")
        else:
            params = rdata["data"]
            params_count = len(params)
           
        chunk_dict = {}
        body_dict = {}

        if params_count <= chunk_size:
            # parameters count is 20 or less, make http request directly
            logging.debug(f"Body passed: {rdata}")
            dev = self.__httpRequest(qpath="setconfig", rtype="POST", ctype=ctype, rdata=rdata)
            if dev != None:
                response.append(dev.json())
        else:
            # need to slice body into chunks of 20 parameters and make http requests by chunks
            item_count = 0
            loop_count = 0

            for k,v in params.items():
                chunk_dict[k] = v
                item_count += 1

                if item_count > chunk_size-1:
                    # chunk collection is full, send for http request, then empty chunk_dict & body_dict
                    item_count = 0
                    loop_count += 1
                    body_dict["data"] = chunk_dict
                    
                    logging.debug(f"Body passed: {body_dict}")
                    dev = self.__httpRequest(qpath="setconfig", rtype="POST", ctype=ctype, rdata=body_dict)
                    if dev != None:
                        response.append(dev.json())
                    
                    chunk_dict.clear()
                    body_dict.clear()

            if params_count%chunk_size != 0:
                # last chunk containing all remainder, send for http request
                body_dict["data"] = chunk_dict
                logging.debug(f"Body passed: {body_dict}")
                dev = self.__httpRequest(qpath="setconfig", rtype="POST", ctype=ctype, rdata=body_dict)
                if dev != None:
                    response.append(dev.json())
            
        return response

    
    def callDial(self, dest, line=1, linetype="Tel", duration=10, ctype="application/json"):
        """
        Method calls internal httpRequest to POST "calldial" : "/api/v1/callctrl/dial". Auto-disconnect is supported for a 
        singe line(1) dialout scenario only.
        INPUTS: 
            dest as string (either in TEL, eg. 3002 or SIP URI format, eg. 3002@apbeta.internal),
            line as int (defaults to line 1), 
            linestype as string (SIP, H323 or TEL, should match dest string format),
                duration as int (in seconds - defaults to 10S,when value is 1s or more (on line1 only), 
                method will track duration and auto-disconnect after duration lapsed.), 0s means no auto-disconnect.
            ctype as string(Content-Type).
        OUTPUT: Returns response body as dict when successful, None when unsuccessful.
        """
        params = {
                    "data" : { 
                                "Dest" : dest,
                                "Line" : line,
                                "Type" : linetype
                             }
                }
        
        logging.debug(f"Body passed: {params}") 
        
        # validates duration as integer
        if not isinstance(duration, int):
            return logging.error("Invalid type for duration, expects Int only")
            
        dev = self.__httpRequest(qpath="calldial", rtype="POST", ctype=ctype, rdata=params)
        if dev != None:
            logging.info(f"Placing call out to '{dest}'...")
            print(f"Placing call out to '{dest}'...")
        
            loop = True
            
            while loop:

                call = self.getCallStatusV2()   
                
                if call != None:
                    if ( len(call["data"] ) == 0):
                        logging.info(f"Call to '{dest}' has ended.")
                        print(f"Call to '{dest}' has ended.")
                        loop = False

                    if( len(call["data"] ) == 1): 
                        # Validates one active connection

                        session_RemotePartyNumber = re.sub( "^sip:", "", call["data"][0]["RemotePartyNumber"] )
                        # Removes 'sip:' from RemotePartyNumber if present.

                        callstate = call["data"][0]["CallState"]

                        logging.info(f"CallState to '{dest}' is currently '{callstate}'.")
                        print(f"CallState to '{dest}' is currently '{callstate}'.")

                        if duration > 0:                       
                            if ( dest == session_RemotePartyNumber ) & ( callstate == "Connected" ):
                                # Validates called party number and connected state before executing callEnd.
                                logging.info(f"Call is now connected to '{dest}'...")
                                print(f"Call is now connected to '{dest}'...")

                                callHandle = call["data"][0]["CallHandle"]
                                time.sleep(duration)
                                logging.info(f"Duration {duration}s has lapsed, attempting to disconnect call now.")
                                print(f"Duration {duration}s has lapsed, attempting to disconnect call now.")

                                res = self.callEnd(callHandle)
                                if res != None:
                                    logging.info(f"Call to '{dest}' has ended.")
                                    print(f"Call to '{dest}' has ended.")
                                else:
                                    logging.info(f"Disconnection attempt to '{dest}' at '{callHandle}' failed. \nPlease check device physically to end the session.")
                                    print(f"Disconnection attempt to '{dest}' at '{callHandle}' failed. \nPlease check device physically to end the session.")

                                return res            
                
                time.sleep(3)
                
            return dev.json()

        
    def callEnd(self, callhandle=None, ctype="application/json"):
        """
        Method calls internal httpRequest to POST "callend" : "/api/v1/callctrl/endCall".
        INPUTS: callHandle as string (should be a valid callHandle from 'getCallStatusV2()'), 
                ctype as string(Content-Type)
        OUTPUT: Returns response body as dict when successful, None when unsuccessful.
        """
        params = {
                    "data" : { 
                                "Ref" : callhandle
                             }
                }
        
        logging.debug(f"Body passed: {params}")
        dev = self.__httpRequest(qpath="callend", rtype="POST", ctype=ctype, rdata=params)
        if dev != None:
            return dev.json()

        
    def simulateKeyEvent(self, key, ctype="application/json"):
        """
        Method calls internal httpRequest to POST "simulateKeyEvent" : "/api/v1/mgmt/simulateKeyEvent".
        INPUTS: key as string (should be a valid KeyName), 
                ctype as string(Content-Type)
        OUTPUT: Returns response body as dict when successful, None when unsuccessful.
        """
        params = {
                    "data" : { 
                                "Type" : "Tap",
                                "KeyName" : key
                             }
                }

        dev = self.__httpRequest(qpath="simulateKeyEvent", rtype="POST", ctype=ctype, rdata=params)
        if dev != None:
            return dev.json()

        
    def simulateTextInput(self, textinput, replacetext="true", ctype="application/json"):
        """
        Method calls internal httpRequest to POST "simulateTextInput" : "/api/v1/mgmt/simulateTextInput".
        INPUTS: textinput as string, 
                replacetext as string ('true' or 'false', if set to true, it replaces any existing text in phone UI’s text field 
                with the value provided.),
                ctype as string(Content-Type)
        OUTPUT: Returns response body as dict when successful, None when unsuccessful.
        """
        params = {
                    "data" : { 
                                "Value" : textinput,
                                "ReplaceText" : replacetext
                             }
                }
        
        logging.debug(f"Body passed: {params}")
        dev = self.__httpRequest(qpath="simulateTextInput", rtype="POST", ctype=ctype, rdata=params)
        if dev != None:
            return dev.json()


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import time
import vvxme
import menu

module_version = "1.0.0.post5"
module_date = "Oct 2019"


# In[ ]:


def main():
    """
    Method is the first main menu.
    INPUTS: none 
    OUTPUT: none
    """
    
    loop = True
    choice = ""
    
    while loop:
        
        menu.clear()
        credentials =[]
        print(f"Welcome to VVXME CLI Menu version {module_version}")
        print("")
        print("1. Connect to a VVX")
        print("0. Exit")
        choice = input("Enter your choice[0-1]: ")
        
        if choice == "1":
            
            print(f"Your choice is {choice}")
            menu.clear()
            
            credentials = menu.connect_device(module_version)
            dev = vvxme.vvx(credentials[0],credentials[1])
                        
            if dev.getDeviceInfoV2() != None:
                menu.selection_menu(dev)
                     
        elif choice == "0":
            print("Goodbye!")
            loop = False
        
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-1] ")
            time.sleep(1)
            menu.clear()


# In[ ]:





# In[ ]:


if __name__ == "__main__":
    main()


# In[ ]:







#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import keyboard
import menu
import time
import sys


# In[ ]:


def simulate_key_events_submenu(dev):
    """
    Method - Simulate Key Events Menu
    INPUTS: dev as object 
    OUTPUT: none
    """
    
    loop = True
    menu.clear()

    while loop:
        print("Simulate Key Events Menu")
        print("========================")
        print("1. Simulate Keys")
        print("0. Exit")
        choice = input("Enter your choice[0-1]: ")
        
        if choice == "1":
            menu.clear()
            simulatekeys_submenu(dev)
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-1].")
            time.sleep(1)
            menu.clear()        


# In[ ]:


def simulatekeys_submenu(dev):
    """
    Method uses keyboard module to capture keyboard event (.is_pressed) and then match to return valid 
    '/api/v1/mgmt/simulateKeyEvent' value. Tested on Win10 and MacOS 10.x. 
    *TAKE NOTE on Mac: Run in sudo mode and allow Accessbility to allow keyboard module to work.
    INPUTS: none 
    OUTPUT: none
    """
    from sys import platform as _platform
    
    menu.clear()
    kb = keyboard
    
    keys_dict = {    
        "up" : "ArrowUp",
        "down" : "ArrowDown",
        "right" : "ArrowRight",
        "left" : "ArrowLeft",
        "0" : "Dialpad0",
        "1" : "Dialpad1",
        "2" : "Dialpad2",
        "3" : "Dialpad3",
        "4" : "Dialpad4",
        "5" : "Dialpad5",
        "6" : "Dialpad6",
        "7" : "Dialpad7",
        "8" : "Dialpad8",
        "9" : "Dialpad9",
        "h" : "Home",
        "H" : "Home",
        "m" : "MicMute",
        "M" : "MicMute",
        "r" : "Redial",
        "R" : "Redial",
        "enter" : "Select",
        "backspace" : "Delete",
        "delete" : "Delete",
        "!" : "SoftKey1",
        "@" : "SoftKey2",
        "#" : "SoftKey3",
        "$" : "SoftKey4",
        "+" : "VolUp",
        "-" : "VolDown"
    }
    
    mac_softkeys_dict = {
        "Softkey1" : [ 56, 18 ],
        "Softkey2" : [ 56, 19 ],
        "Softkey3" : [ 56, 20 ],
        "Softkey4" : [ 56, 21 ]
    }
    
    print("Keyboard Legend:")
    print("================")
    print("Arrows:[up, down, left, right], Numbers:[0-9]") 
    print("Letters:[(h)ome, (m)icMute, (r)edial, (e)nter, backspace/delete]")
    print("Softkeys(1,2,3,4): [!, @, #, $], Volume: [+,-]")
    print("\nPress 'esc' to exit.\n")
     
        
    def print_pressed_keys(e):
        
        if ( _platform == "win32" ) | ( _platform == "win32" ): 
            # Windows
            keys = kb._pressed_events 
            for item in list(keys):
                if keys[item].name in keys_dict:
                    pressed_key = keys_dict[keys[item].name]                
                    print(f"Keyboard code pressed: {item}, translated to '{pressed_key}' for VVX", end='\r')
                    sys.stdout.write("\033[K")  # clears previous print() line
                    dev.simulateKeyEvent(pressed_key)
    
        elif _platform == "darwin":
            # MacOS
            keys = kb._pressed_events           
            if len(keys) == 1: 
                # single input
                for item in list(keys):
                    if keys[item].name in keys_dict:
                        pressed_key = keys_dict[keys[item].name]                
                        print(f"Keyboard code pressed: {item}, translated to '{pressed_key}' for VVX", end='\r')
                        sys.stdout.write("\033[K")  # clears previous print() line
                        dev.simulateKeyEvent(pressed_key)
            elif len(keys) == 2:
                # double inputs - looking for shift + 1,2,3,4
                list_keys = list(keys)
                for k, v in mac_softkeys_dict.items():
                    if ( list_keys[0] in v ) & ( list_keys[1] in v ):
                        print(f"Keyboard code pressed: {v}, translated to '{k}' for VVX", end='\r')
                        sys.stdout.write("\033[K")  # clears previous print() line
                        dev.simulateKeyEvent(k)               
    
    kb.hook(print_pressed_keys)
    kb.wait('esc')
    keyboard.unhook_all()
    menu.flush_input()




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import menu
import time


# In[ ]:


def web_call_controls_submenu(dev):
    """
    Method - Web Call Control Menu
    INPUTS: dev as object 
    OUTPUT: none
    """

    loop = True
    menu.clear()

    while loop:
        print("Web Call Control Menu")
        print("=====================")
        print("1. Call Dial")
        print("0. Exit")
        choice = input("Enter your choice[0-1]: ")
        
        if choice == "1":
            menu.clear()
            calldial_submenu(dev)
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-1].")
            time.sleep(1)
            menu.clear()        


# In[ ]:


def calldial_submenu(dev):
    
    menu.clear()
    
    linetype = "TEL"
    line = 1
    dest = ""
    duration = 0
    
    print("Current Call Dial Settings")
    print("==========================")
    print(f">> Line Type: [ {linetype} ] \n>> Line: [ {line} ] \n>> Dest: [ {dest} ] \n>> Duration: [ {duration} ]\n")
        
                         
    linetype = menu.getlinetype_input()
    line = menu.getline_input()
    dest = menu.getdest_input(linetype)
    duration = menu.getduration_input()
     
    #print(f"Final Call Dial Settings >> Line Type: [{linetype}], Line: [{line}], Dest: [{dest}], Duration: [{duration}]\n")
    res = dev.callDial(dest, line, linetype, duration)
    print(res)
    input("Press Enter to continue...")
    




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import menu
import time


# In[ ]:


def quick_configuration_submenu(dev):
    """
    Method - Quick Configuration Menu
    INPUTS: dev as object 
    OUTPUT: none
    """
    
    loop = True
    menu.clear()
    
    while loop:
        
        print("Quick Configuration Menu")
        print("========================")        
        print("1. Acoustic Fence Menu")
        print("2. Active Call Screen")
        print("3. Font Size Customization")
        print("4. Headset Configuration Menu")
        print("5. Message Waiting Indicator(LED)")
        print("6. Pagination")
        print("7. Remote Packet Capture")
        print("8. Screen Capture")
        print("9. SIP Autoanswer")
        print("10. SIP Debug & USB Logging")
        print("0. Exit")  
        choice = input("Enter your choice[0-10]: ")
        
        if choice == "1":
            # Calls acousticfencemenu_submenu
            menu.clear()
            acousticfencemenu_submenu(dev)
            menu.clear()
        elif choice == "2":
            # Calls activecallscreen_submenu
            menu.clear()
            activecallscreen_submenu(dev)
            menu.clear()
        elif choice == "3":
            # Calls fontsizecustomization_submenu
            menu.clear()
            fontsizecustomization_submenu(dev)
            menu.clear()
        elif choice == "4":
            # Calls headsetconfiguration_submenu
            menu.clear()
            headsetconfiguration_submenu(dev)
            menu.clear()
        elif choice == "5":
            # Calls messagewaitingindicator_submenu
            menu.clear()
            messagewaitingindicator_submenu(dev)
            menu.clear()
        elif choice == "6":
            # Calls pagination_submenu
            menu.clear()
            pagination_submenu(dev)
            menu.clear()   
        elif choice == "7":
            # Calls remotepacketcapture_submenu
            menu.clear()
            remotepacketcapture_submenu(dev)
            menu.clear()
        elif choice == "8":
            # Calls screencapture_submenu
            menu.clear()
            screencapture_submenu(dev)
            menu.clear()
        elif choice == "9":
            # Calls sipautoanswer_submenu
            menu.clear()
            sipautoanswer_submenu(dev)
            menu.clear()
        elif choice == "10":
            # Calls sipdebugusblogging_submenu
            menu.clear()
            sipdebugusblogging_submenu(dev)
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-10].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def acousticfencemenu_submenu(dev):
   
    menu.clear()
    query_dict = { "data" : ["feature.acousticFenceUI.enabled"] }
    body_dict = { "data" : {} }
    loop = True
        
    while loop:
        
        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable Acoustic Fence Menu")
        print("2. Disable Acoustic Fence Menu")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["feature.acousticFenceUI.enabled"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["feature.acousticFenceUI.enabled"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def activecallscreen_submenu(dev):
   
    menu.clear()
    query_dict = { "data" : ["up.LineViewCallStatus.enabled"] }
    body_dict = { "data" : {} }
    loop = True
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable Active Call Screen")
        print("2. Disable Active Call Screen")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["up.LineViewCallStatus.enabled"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["up.LineViewCallStatus.enabled"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def fontsizecustomization_submenu(dev):
   
    menu.clear()
    valid_models = ("VVX 250", "VVX 350", "VVX 450")
    query_dict = { "data" : ["device.font.size"] }
    body_dict = { "data" : {} }
    loop = True
    
    if dev.model not in valid_models:
        print(f"This device {dev.model} doesn't support this setting. Supported models are {valid_models}")
        input("Press Enter to continue...")
        return
    
    if not dev._swVer:
        print(f"This firmware {dev.firmware} doesn't support this setting. Supported versions are {dev._valid_versions}")
        input("Press Enter to continue...")
        return
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("***NOTE: Changing this value will cause device to reboot!***")
        print("")
        print("1. Normal Font Size")
        print("2. Large Font Size")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.font.size"] = "Normal"
            body_dict["data"]["device.font.size.set"] = "1"
            print(dev.setConfig(body_dict))
            print("Device is rebooting, pausing for 30sec...")
            time.sleep(30)
            #input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.font.size"] = "Large"
            body_dict["data"]["device.font.size.set"] = "1"
            print(dev.setConfig(body_dict))
            print("Device is rebooting, pausing for 30sec...")
            time.sleep(30)
            #input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def headsetconfiguration_submenu(dev):
   
    menu.clear()
    query_dict = { "data" : ["usb.headset.config.enabled"] }
    body_dict = { "data" : {} }
    loop = True

    if not dev._swVer:
        print(f"This firmware {dev.firmware} doesn't support this setting. Supported versions are {dev._valid_versions}")
        input("Press Enter to continue...")
        return
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable Headset Configuration Menu")
        print("2. Disable Headset Configuration Menu")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["usb.headset.config.enabled"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["usb.headset.config.enabled"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def messagewaitingindicator_submenu(dev, line=1):
   
    menu.clear()
    body_dict = { "data" : {} }
    loop = True

    while loop:

        print("Current Value:")
        print("==============")
        for i in range (dev.linescount):
            query_dict = { "data" : [f"msg.mwi.{i+1}.led"] }
            menu.display_dict(dev.getConfig(query_dict)["data"])
        print("***NOTE: Changing this value will cause device to restart!***")
        print("")
        print("1. Enable Message Waiting Indicator(LED)")
        print("2. Disable Message Waiting Indicator(LED)")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            line = menu.getline_input()
            body_dict["data"][f"msg.mwi.{line}.led"] = "1"
            print(dev.setConfig(body_dict))
            print("Device is restarting, pausing for 30sec...")
            time.sleep(30)
            #input("Press Enter to continue...")
            return
        elif choice == "2":
            line = menu.getline_input()
            body_dict["data"][f"msg.mwi.{line}.led"] = "0"
            print(dev.setConfig(body_dict))
            print("Device is restarting, pausing for 30sec...")
            time.sleep(30)
            #input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def pagination_submenu(dev):
   
    menu.clear()
    invalid_models = ("VVX 101", "VVX 201", "VVX 150")
    query_dict = { "data" : ["up.Pagination.enabled"] }
    body_dict = { "data" : {} }
    loop = True
    
    if dev.model in invalid_models:
        print(f"This device {dev.model} doesn't support this setting. Unsupported models are {invalid_models}")
        input("Press Enter to continue...")
        return

    if not dev._swVer:
        print(f"This firmware {dev.firmware} doesn't support this setting. Supported versions are {dev._valid_versions}")
        input("Press Enter to continue...")
        return
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable Pagination")
        print("2. Disable Pagination")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["up.Pagination.enabled"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["up.Pagination.enabled"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def remotepacketcapture_submenu(dev):
   
    menu.clear()
    query_dict = { "data" : ["diags.pcap.enabled", "diags.pcap.remote.enabled"] }
    body_dict = { "data" : {} }
    loop = True
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable Remote Packet Capture")
        print("2. Disable Remote Packet Capture")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["diags.pcap.enabled"] = "1"
            body_dict["data"]["diags.pcap.remote.enabled"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["diags.pcap.enabled"] = "0"
            body_dict["data"]["diags.pcap.remote.enabled"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def screencapture_submenu(dev):
   
    menu.clear()
    query_dict = { "data" : ["up.screenCapture.enabled", "up.screenCapture.value"] }
    body_dict = { "data" : {} }
    loop = True
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable Screen Capture")
        print("2. Disable Screen Capture")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["up.screenCapture.enabled"] = "1"
            body_dict["data"]["up.screenCapture.value"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["up.screenCapture.enabled"] = "0"
            body_dict["data"]["up.screenCapture.value"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def sipautoanswer_submenu(dev):
    
    menu.clear()
    query_dict = { "data" : ["call.autoAnswer.SIP"] }
    body_dict = { "data" : {} }
    loop = True
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable SIP Autoanswer")
        print("2. Disable SIP Autoanswer")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["call.autoAnswer.SIP"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["call.autoAnswer.SIP"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def sipdebugusblogging_submenu(dev):
   
    menu.clear()
    invalid_models = ("VVX 101", "VVX 201", "VVX 301", "VVX 311", "VVX 150")
    query_dict = { "data" : ["log.level.change.sip", "log.render.level", "log.render.file.size", "feature.usbLogging.enabled"] }
    body_dict = { "data" : {} }
    loop = True
    
    if dev.model in invalid_models:
        print(f"This device {dev.model} doesn't support USB. Unsupported models are {invalid_models}")
        input("Press Enter to continue...")
        return
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable SIP Debug & USB Logging")
        print("2. Disable SIP Debug & USB Logging")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["log.render.level"] = "0"
            body_dict["data"]["log.render.file.size"] = "1"
            body_dict["data"]["log.level.change.sip"] = "0"
            body_dict["data"]["feature.usbLogging.enabled"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["log.render.level"] = "4"
            body_dict["data"]["log.render.file.size"] = "32"
            body_dict["data"]["log.level.change.sip"] = "4"
            body_dict["data"]["feature.usbLogging.enabled"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def networkcdp_submenu(dev):
    # not in use...

    menu.clear()
    query_dict = { "data" : ["device.net.cdpEnabled"] }
    body_dict = { "data" : {} }
    loop = True
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable CDP")
        print("2. Disable CDP")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.net.cdpEnabled"] = "1"
            body_dict["data"]["device.net.cdpEnabled.set"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.net.cdpEnabled"] = "0"
            body_dict["data"]["device.net.cdpEnabled.set"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def networklldp_submenu(dev):
    # not in use...
   
    menu.clear()
    query_dict = { "data" : ["device.net.lldpEnabled"] }
    body_dict = { "data" : {} }
    loop = True
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("")
        print("1. Enable LLDP")
        print("2. Disable LLDP")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.net.lldpEnabled"] = "1"
            body_dict["data"]["device.net.lldpEnabled.set"] = "1"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.net.lldpEnabled"] = "0"
            body_dict["data"]["device.net.lldpEnabled.set"] = "0"
            print(dev.setConfig(body_dict))
            input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            


# In[ ]:


def phonetheme_submenu(dev):
    # not in use...
   
    menu.clear()
    invalid_models = ("VVX 101", "VVX 201", "VVX 150")
    query_dict = { "data" : ["device.theme"] }
    body_dict = { "data" : {} }
    loop = True
    
    if dev.model in invalid_models:
        print(f"This device {dev.model} doesn't support this setting. Unsupported models are {invalid_models}")
        input("Press Enter to continue...")
        return
    
    while loop:

        print("Current Value:")
        print("==============")
        menu.display_dict(dev.getConfig(query_dict)["data"])
        print("***NOTE: Changing this value will cause device to reboot!***")
        print("")
        print("1. Classic Theme")
        print("2. BroadSoft Theme")
        print("0. Exit")
        choice = input("Enter your choice[0-2]: ")
        
        if choice == "1":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.theme"] = "Classic"
            body_dict["data"]["device.theme.set"] = "1"
            print(dev.setConfig(body_dict))
            print("Device is rebooting, pausing for 30sec...")
            time.sleep(30)
            #input("Press Enter to continue...")
            return
        elif choice == "2":
            # Calls setConfig using body_dict
            body_dict["data"]["device.set"] = "1"
            body_dict["data"]["device.theme"] = "BroadSoft"
            body_dict["data"]["device.theme.set"] = "1"
            print(dev.setConfig(body_dict))
            print("Device is rebooting, pausing for 30sec...")
            time.sleep(30)
            #input("Press Enter to continue...")
            return              
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-2].")
            time.sleep(1)
            menu.clear()
            




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import menu
import time


# In[ ]:


def device_infomation_submenu(dev):  
    """
    Method - Show Device Information Menu
    INPUTS: dev as object 
    OUTPUT: none
    """
    
    loop = True
    menu.clear()

    while loop:
        print("Device Information Menu")
        print("=======================")
        print("1. Show Device Info")
        print("2. Show Device Stats")
        print("3. Show Network Info")
        print("4. Show Network Stats")
        print("5. Show Line Info")
        print("6. Show Running Config")
        print("7. Show Session Stats")
        print("8. Show Call Logs")
        print("0. Exit")
        choice = input("Enter your choice[0-7]: ")

        if choice == "1":
            ## Show device info
            menu.clear()
            print(f"Device Info - {dev._qpaths_dict['deviceinfov2']}")
            print("======================================")
            menu.display_dict(dev.getDeviceInfoV2()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "2":
            # Show device stats
            menu.clear()
            print(f"Device Stats - {dev._qpaths_dict['devicestats']}")
            print("========================================")
            menu.display_dict(dev.getDeviceStats()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "3":
            # Show network info
            menu.clear()
            print(f"Network Info - {dev._qpaths_dict['network']}")
            print("========================================")
            menu.display_dict(dev.getNetwork()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "4":
            # Show network stats
            menu.clear()
            print(f"Network Stats - {dev._qpaths_dict['networkstats']}")
            print("==========================================")
            menu.display_dict(dev.getNetworkStats()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "5":
            # Show line info
            menu.clear()
            print(f"Line Info - {dev._qpaths_dict['lineinfov2']}")
            print("=================================")            
            menu.display_dict(dev.getLineInfoV2()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()        
        elif choice == "6":
            # Show Running info
            menu.clear()
            print(f"Running Config - {dev._qpaths_dict['runningConfig']}")
            print("==================================================")            
            menu.display_dict(dev.getRunningConfig()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "7":
            # Show session stats
            menu.clear()
            print(f"Session Stats - {dev._qpaths_dict['sessionStats']}")
            print("===============================================")            
            menu.display_dict(dev.getSessionStats()["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()        
        elif choice == "8":
            # Show call logs 
            menu.clear()      
            calllogs_submenu(dev)
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-7].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def calllogs_submenu(dev):  
    """
    Method - Show Call Logs Menu
    INPUTS: dev as object 
    OUTPUT: none
    """
    
    loop = True
    menu.clear()

    while loop:
        print("Call Logs Menu")
        print("==============")
        print("1. Show Missed Calls")
        print("2. Show Received Calls")
        print("3. Show Placed Calls")
        print("0. Exit")
        choice = input("Enter your choice[0-3]: ")

        if choice == "1":
            ## Show missed calls
            menu.clear()
            print(f"Missed Calls - {dev._qpaths_dict['callLogs_missed']}")
            print("===========================================")
            menu.display_dict(dev.getCallLogs("missed")["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "2":
            # Show received calls
            menu.clear()
            print(f"Received Calls - {dev._qpaths_dict['callLogs_received']}")
            print("===============================================")
            menu.display_dict(dev.getCallLogs("received")["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "3":
            # Show placed calls
            menu.clear()
            print(f"Placed Calls - {dev._qpaths_dict['callLogs_placed']}")
            print("===========================================")
            menu.display_dict(dev.getCallLogs("placed")["data"])
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-3].")
            time.sleep(1)
            menu.clear()




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import menu
import time


# In[ ]:


def selection_menu(dev):
    """
    Method is the main selection menu, after successful connectivity to device.
    INPUTS: dev as object 
    OUTPUT: none
    """
    
    loop = True
    menu.clear()
       
    while loop:
        
        print(f"Connected to device {dev.model} with MAC {dev.macaddress} running on build {dev.firmware} in {dev.baseprofile} mode.")
        
        for i in range(dev.linescount):
            print(f">> Line {i+1} configured as {dev.lines[i+1]} in [{dev.linestates[dev.lines[i+1]]}] state")
        
        print("")
        print("Main Selection Menu")
        print("===================")
        print("1. Device Information Menu")
        print("2. Device Management Menu")
        print("3. Quick Configuration Menu") 
        print("4. Web Call Controls Menu")
        print("5. Simulate Key Events Menu")
        print("0. Exit")  
        choice = input("Enter your choice[0-5]: ")
        
        if choice == "1":
            # Calls device_infomation_submenu
            menu.clear()
            menu.device_infomation_submenu(dev)
            menu.clear()
        elif choice == "2":
            # Calls device_management_submenu
            menu.clear()
            menu.device_management_submenu(dev)
            menu.clear()
        elif choice == "3":
            # Calls quick_configuration_submenu
            menu.clear()
            menu.quick_configuration_submenu(dev)
            menu.clear()
        elif choice == "4":
            # Calls web_call_controls_submenu
            menu.clear()
            menu.web_call_controls_submenu(dev)
            menu.clear()
        elif choice == "5":
            # Calls simulate_key_events_submenu
            menu.clear()
            menu.simulate_key_events_submenu(dev)
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-5].")
            time.sleep(1)
            menu.clear()




#!/usr/bin/env python
# coding: utf-8

# In[ ]:


import menu
import time


# In[ ]:


def device_management_submenu(dev):  
    """
    Method - Show Device Information Menu
    INPUTS: dev as object 
    OUTPUT: none
    """
    loop = True
    menu.clear()

    while loop:
        print("Device Management Menu")
        print("======================")
        print("1. Import from cfg (xml/json)")
        print("2. Reset Configuration")
        print("3. Update Configuration")
        print("4. Restart Device")
        print("5. Reboot Device")
        print("6. Factory Reset")
        print("0. Exit")
        choice = input("Enter your choice[0-6]: ")

        if choice == "1":
            # Import from cfg
            menu.clear()
            importconfiguration_submenu(dev)
            menu.clear()
        elif choice == "2":
            # Reset Configuration
            menu.clear()
            resetconfig_submenu(dev)
            menu.clear()
        elif choice == "3":
            # Update Configuration
            menu.clear()
            print(f"Update Configuration - {dev._qpaths_dict['updateConfig']}")
            print("=======================================================")
            res = menu.getconfirmation_input("Update Configuration")
            if res:
                print("Updating Configuration...")
                print(dev.updateConfig())
            else:
                print("\nNo action taken.")            
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "4":
            # Restart Device
            menu.clear()
            print(f"Restart Device - {dev._qpaths_dict['safeRestart']}")
            print("=========================================")
            res = menu.getconfirmation_input("Restart Device")
            if res:
                print("Restarting Device...")
                print(dev.safeRestart())
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "5":
            # Reboot Device
            menu.clear()
            print(f"Reboot Device - {dev._qpaths_dict['safeReboot']}")
            print("=======================================")
            res = menu.getconfirmation_input("Reboot Device")
            if res:
                print("Rebooting Device...")
                print(dev.safeReboot())
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "6":
            # Factory Reset Device
            menu.clear()
            print(f"Factory Reset - {dev._qpaths_dict['factoryReset']}")
            print("=========================================")
            res = menu.getconfirmation_input("Factory Reset Device")
            if res:
                print("Factory Resetting Device...")
                print(dev.factoryReset())
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-6].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def resetconfig_submenu(dev):  
    """
    Method - Show Reset Config Menu
    INPUTS: dev as object 
    OUTPUT: none
    """
    
    loop = True
    menu.clear()

    while loop:
        print("Reset Configuration Menu")
        print("========================")
        print("1. Reset All")
        print("2. Reset Cloud")
        print("3. Reset Local")
        print("4. Reset Web")
        print("5. Reset Device")
        print("0. Exit")
        choice = input("Enter your choice[0-5]: ")

        if choice == "1":
            ## Reset all
            menu.clear()
            print(f"Reset Configuration - {dev._qpaths_dict['resetConfig']}")
            print("==============================================")
            res = menu.getconfirmation_input("Reset All configuration")
            if res:
                print("Resetting ALL configuration...")
                print(dev.resetConfig())
            else:
                print("\nNo action taken.")
            print("")                  
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "2":
            ## Reset cloud
            menu.clear()
            print(f"Reset Configuration - {dev._qpaths_dict['resetConfig_cloud']}")
            print("====================================================")
            res = menu.getconfirmation_input("Reset Cloud configuration")
            if res:
                print("Resetting Cloud configuration...")
                print(dev.resetConfig("cloud"))
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "3":
            ## Reset local
            menu.clear()
            print(f"Reset Configuration - {dev._qpaths_dict['resetConfig_local']}")
            print("====================================================")
            res = menu.getconfirmation_input("Reset Local configuration")
            if res:
                print("Resetting Local configuration...")
                print(dev.resetConfig("local"))
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "4":
            ## Reset web
            menu.clear()
            print(f"Reset Configuration - {dev._qpaths_dict['resetConfig_web']}")
            print("==================================================")
            res = menu.getconfirmation_input("Reset Web configuration")
            if res:
                print("Resetting Web configuration...")
                print(dev.resetConfig("web"))
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "5":
            ## Reset web
            menu.clear()
            print(f"Reset Configuration - {dev._qpaths_dict['resetConfig_device']}")
            print("=====================================================")
            res = menu.getconfirmation_input("Reset Device configuration")
            if res:
                print("Resetting Device configuration...")
                print(dev.resetConfig("device"))
            else:
                print("\nNo action taken.")
            print("")
            input("Press Enter to continue...")
            menu.clear()
        elif choice == "0":
            # Exit menu
            loop = False
        else:
            print(f"Invalid input '{choice}' >> Expecting [0-5].")
            time.sleep(1)
            menu.clear()


# In[ ]:


def importconfiguration_submenu(dev):
    """
    Method - Show the parsed content and ask user confirmation for import.
    INPUTS: none 
    OUTPUT: none
    """
    import os.path
    
    body_dict = {}
    loop = True
    menu.clear()
    
    while loop:
        
        print("*** Quick Note #1 : Expecting configuration file in current folder only. ***")
        print("*** Quick Note #2 : Parser will attempt to parse with xml first, and then with json. ***")
        print("*** Quick Note #3 : xml - uses VVX/UCS syntax, Root Element is not parsed! ***")
        print("*** Quick Note #4 : json - { 'data' : { 'parameter1' : 'value1', 'parameter2' : 'value2', ... } } ***")
        print("*** Quick Note #5 : It is ultimately your responsibilty to ensure correctness of your file!!! ***\n")
        filename = input("Enter filename (default='import.cfg'),or E(x)it: ")
        
        if (filename == 'x') | (filename == 'X'):
            return
        
        if filename == "":
            filename = "import.cfg"
        
        if os.path.isfile(filename):                   
            body_dict = menu.configfile_parser(filename)

            if body_dict != None:
                
                import_loop = True
                menu.display_dict(body_dict)
                
                while import_loop:
                
                    import_proceed = input("\nPlease review parsed parameters. Proceed with import [Y/n]: ")

                    if (import_proceed == "") | (import_proceed == "Y") | (import_proceed == "y"):
                        print(dev.setConfig(body_dict))
                        print("\n**Device may reboot after import**\n")
                        input("Press Enter to continue...")
                        return
                    elif (import_proceed == "N") | (import_proceed == "n"):
                        print("Import did not proceed!")
                        input("Press Enter to continue...")
                        return
                    else:
                        print(f"Invalid input '{import_proceed}' >> Expecting [y/Y/n/N].")
                        time.sleep(1)
                
            else:
                menu.clear()
        
        else:
            print(f"\nFile Not Found: '{filename}'. Please try again...\n")
            input("Press Enter to continue...")
            menu.clear()
    
    




#!/usr/bin/env python
# coding: utf-8

# In[8]:


def clear(): 
    """
    Method clears the terminal screen.
    INPUTS: none
    OUTPUT: none
    """
    from os import system, name 
    
    # for windows 
    if name == 'nt': 
        _ = system('cls') 
  
    # for mac and linux(here, os.name is 'posix') 
    else: 
        _ = system('clear') 
         


# In[ ]:


def flush_input():
    """
    Method attempts to flush keyboard inputs.
    INPUTS: none
    OUTPUT: none
    """
    try:
        import msvcrt
        while msvcrt.kbhit():
            msvcrt.getch()
    except ImportError:
        import sys, termios
        termios.tcflush(sys.stdin, termios.TCIOFLUSH)


# In[9]:


def display_dict(dev, indent_count=0):
    """
    Method recursively prints data body containing either dict or list
    INPUTS: none 
    OUTPUT: none
    """
    
    indent = "\t"
    
    if isinstance(dev, dict): 
        
        for k, v in dev.items():
            if isinstance(v, dict):
                if indent_count == 0:
                    print(f"{k}:")
                else:
                    print(f"{indent_count*indent}{k}:")
                
                display_dict(v, indent_count+1)
                
            elif isinstance(v, list):
                if indent == 0:
                    print(f"{k}:")
                else:
                    print(f"{indent_count*indent}{k}:")
                    
                display_dict(v, indent_count+1)
                
            else:
                if indent == 0:
                    print(f"{k} = {v}")
                else:
                    print(f"{indent_count*indent}{k} = {v}")
    
    elif isinstance(dev, list):
        
        for i in range( len(dev) ):
            if indent == 0:
                display_dict(dev[i])
            else:
                display_dict(dev[i], indent_count+1)
        


# In[10]:


def getline_input():
    """
    Method collects user inputs for phone line number 
    INPUTS: none 
    OUTPUT: Returns int
    """
   
    loop = True
    
    while loop:
        
        line = input("Enter line no. (default=1): ")
        
        if line == "":
            # empty input defaults to 1.
            return 1
        
        if line.isdigit(): 
            if int(line) <= 0:
                print(f"Invalid input '{line}' >> Expecting integer(>=1).")
            else:
                loop = False
        else:
            print(f"Invalid input '{line}' >> Expecting integer(>=1).")            
    
    return int(line)
    


# In[11]:


def getlinetype_input():
    """
    Method collects user inputs for line type - TEL or SIP 
    INPUTS: none 
    OUTPUT: Returns str
    """
    
    valid_values = ( "TEL", "SIP" )
    loop = True
    
    while loop:
        
        linetype = input(f"Enter line type, {valid_values} (default='TEL'): ")
        
        if linetype == "":
            # empty input defaults to 1.
            return "TEL"
        
        if linetype.upper() in valid_values: 
            loop = False
        else:
            print(f"Invalid input '{linetype}' >> Expecting [{valid_values}].")            
    
    return linetype.upper()    
    


# In[12]:


def getdest_input(linetype="TEL"):
    """
    Method collects user inputs for destination dialstring - TEL or SIP 
    INPUTS: linetype as str ('TEL' or 'SIP') 
    OUTPUT: Returns str
    """
    import re
       
    loop = True
    
    while loop:
        
        dest = input(f"Enter destination dialstring [{linetype}]: ")
        
        if dest != "":
            
            if linetype == "TEL":
                res = re.match("\d+", dest)
                if res:
                    loop = False
                else:
                    print(f"Invalid input '{dest}' >> Expecting TEL(E.164) dialstring.")
                    
            elif linetype == "SIP":
                res = re.match("([^@|\s]+@[^@]+\.[^@|\s]+)", dest)
                if res:
                    loop = False
                else:
                    print(f"Invalid input '{dest}' >> Expecting SIP URI dialstring.")
                
        else:
            print(f"Empty input '{dest}' >> Expecting {linetype} dialstring.")
    
    return dest
    


# In[ ]:


def getduration_input():
    """
    Method collects user inputs for duration in seconds, minimum value is 0 and maximum value is 600s. Defaults to 0 
    if not inputs from user.
    INPUTS: none 
    OUTPUT: Returns int. 
    """
   
    loop = True
    
    while loop:
        
        duration = input("Enter call duration(s) (default=10): ")
        
        if duration == "":
            # empty input defaults to 10.
            return 10
        
        if duration.isdigit(): 
            if ( int(duration) < 0 ) | ( int(duration) > 600 ):
                print(f"Invalid input '{duration}' >> Expecting integer(0-600).")
            else:
                loop = False
        else:
            print(f"Invalid input '{duration}' >> Expecting integer.")            
    
    return int(duration)
    


# In[ ]:


def getconfirmation_input(action):
    """
    Method collects user confirmation to proceed with action 
    INPUTS: action as str, description of the action 
    OUTPUT: Returns boolean, True to proceed, False to not proceed.
    """
    loop = True
    
    while loop:
        
        user_input = input(f"Confirm to proceed with '{action}'? [y/N]: ")
        
        if (user_input == "Y") | (user_input == "y"):
            return True
        elif (user_input == "") | (user_input == "N") | (user_input == "n"): 
            return False
        else:
            print(f"Invalid input '{user_input}' >> Expecting [y/Y/n/N].")                
    


# In[ ]:


def connect_device(module_version, username="Polycom", password="789"):
    """
    Method collects user inputs for phone's IP address, username and password
    INPUTS: none 
    OUTPUT: Returns result as list, [ ip, (username, passwd) ]
    """
    import ipaddress
    
    ip = ""
    loop = True
    
    print(f"Welcome to VVXME CLI Menu version {module_version}\n")

    while loop:            
        try:
            ip = ipaddress.ip_address( input("Enter IP address of the phone: ") )
            print(f"{ip} is a valid IP{ip.version} address")
            loop = False
        except ValueError as err:
            print(f"Value Error: <{err}>")


    usr = input(f"Enter username(Default: '{username}') :")
    if usr != "":
        username = usr

    pwd = input(f"Enter password(Default: '{password}'): ")
    if pwd != "":
        password = pwd

    return [ip.exploded, (username, password)]


# In[ ]:


def configfile_parser(filename="import.cfg"):
    """
    Method attempts to parse input 'filename' as XML first. Upon XML parse failure, attempt next to parse as JSON.
    Method doesn't cater for absolute path+name, expects 'filename' to exists in current folder only.
    XML Body: uses standard VVX/UCS syntax. Does not parse Root Element.
    JSON Body: expects UCS RestAPI body, eg. { 'data' : { 'parameter1' : 'value1', 'parameter2' : 'value2', ... } }
    INPUTS: filename as str, expect to be existent no further filename checks.
    OUTPUT: Returns result as dict when parse is successful, None when parse fails.
    """
    
    import json
    import xml.etree.ElementTree as ET
    import time
    
    params_dict = {}
    body_dict = {}
    
    try:
        tree = ET.parse(filename)
        root = tree.getroot()
        
        for child in root:
            for sub in child.iter():
                if sub.attrib:
                    for k, v in sub.attrib.items():
                        params_dict[k] = v
        
        body_dict["data"] = params_dict
        print("")
        print("Parse XML Success!")
        print("==================")
        return body_dict
           
    except ET.ParseError as xml_err:
        print(f"[XML] Parse Error: <{xml_err}>\n")
        print("Trying to parse JSON now...\n")
        #input("Press Enter to continue...")

        try:
            with open(filename, 'r') as f:
                params_dict = json.load(f)
                body_dict = params_dict
                print("")
                print("Parse JSON Success!")
                print("===================")
                return body_dict
                
        except ValueError as json_err:
            print(f"[JSON] Parse Error: <{json_err}>\n") 
            print("Failed parse JSON too. Please try again...\n")
            input("Press Enter to continue...")
            return
            




#!/usr/bin/env python
# coding: utf-8

# In[4]:


__all__ = [ "clear", 
            "flush_input", 
            "display_dict", 
            "getline_input", 
            "getlinetype_input", 
            "getdest_input", 
            "getduration_input", 
            "getconfirmation_input", 
            "connect_device", 
            "configfile_parser", 
            "selection_menu", 
            "device_infomation_submenu", 
            "calllogs_submenu", 
            "device_management_submenu", 
            "resetconfig_submenu", 
            "importconfiguration_submenu", 
            "quick_configuration_submenu", 
            "acousticfencemenu_submenu", 
            "activecallscreen_submenu", 
            "fontsizecustomization_submenu", 
            "headsetconfiguration_submenu", 
            "messagewaitingindicator_submenu", 
             "pagination_submenu", 
            "remotepacketcapture_submenu", 
            "screencapture_submenu", 
            "sipautoanswer_submenu", 
            "sipdebugusblogging_submenu", 
            "web_call_controls_submenu", 
            "calldial_submenu", 
            "simulate_key_events_submenu", 
            "simulatekeys_submenu" ]


# In[ ]:


from .menu_commons import clear
from .menu_commons import flush_input
from .menu_commons import display_dict
from .menu_commons import getline_input
from .menu_commons import getlinetype_input
from .menu_commons import getdest_input
from .menu_commons import getduration_input
from .menu_commons import getconfirmation_input
from .menu_commons import connect_device
from .menu_commons import configfile_parser
from .selection_menu import selection_menu
from .device_infomation_submenu import device_infomation_submenu
from .device_infomation_submenu import calllogs_submenu
from .device_management_submenu import device_management_submenu
from .device_management_submenu import resetconfig_submenu
from .device_management_submenu import importconfiguration_submenu
from .quick_configuration_submenu import quick_configuration_submenu
from .quick_configuration_submenu import acousticfencemenu_submenu
from .quick_configuration_submenu import activecallscreen_submenu
from .quick_configuration_submenu import fontsizecustomization_submenu
from .quick_configuration_submenu import headsetconfiguration_submenu
from .quick_configuration_submenu import messagewaitingindicator_submenu
from .quick_configuration_submenu import pagination_submenu
from .quick_configuration_submenu import remotepacketcapture_submenu
from .quick_configuration_submenu import screencapture_submenu
from .quick_configuration_submenu import sipautoanswer_submenu
from .quick_configuration_submenu import sipdebugusblogging_submenu
from .web_call_controls_submenu import web_call_controls_submenu
from .web_call_controls_submenu import calldial_submenu
from .simulate_key_events_submenu import simulate_key_events_submenu
from .simulate_key_events_submenu import simulatekeys_submenu


# In[ ]:




